/*
binclk Version 0.6.0 - Binary Clock
Copyright (C) 2001-2010  dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <ncurses.h>
#include <time.h>
#include <sys/times.h>

int alfa,runflg;
int prevyr,prevmon,prevday,prevjul,prevdow;
int prevhr,prevmin,prevsec,prevdst;

void initnc()
   {
   initscr();
   cbreak();
   noecho();
   nonl();
   intrflush(stdscr,FALSE);
   keypad(stdscr,TRUE);
   nodelay(stdscr,TRUE);
   curs_set(0);
   clear();
   refresh();
   } /* initnc */

void escape()
   {
   int ch;
   ch = getch();
   if (ch == 0x1b)
      {
      runflg = 0;
      }
   else if (ch == 'a')
      {
      alfa = (alfa ^ 1) & 1;
      prevyr = prevjul = prevdst = -99999;
      move(9,0);
      clrtoeol();
      move(10,0);
      clrtoeol();
      move(11,0);
      clrtoeol();
      move(12,0);
      clrtoeol();
      } /* if toggle alfa */
   } /* escape */

int iflpyr(int year)
   {
   if (year < 1583) return(0);
   if (!(year % 400)) return(1);
   else if (!(year % 100)) return(0);
   else if (!(year % 4)) return(1);
   else return(0);
   } /* iflpyr */

void putyear(int year)
   {
   int iflpyr();
   if (year & 0x0800)
      mvaddch(0,10,'o');
   else mvaddch(0,10,'.');
   if (year & 0x0400)
      addch('o');
   else addch('.');
   if (year & 0x0200)
      addch('o');
   else addch('.');
   if (year & 0x0100)
      addch('o');
   else addch('.');
   if (year & 0x0080)
      addch('o');
   else addch('.');
   if (year & 0x0040)
      addch('o');
   else addch('.');
   if (year & 0x0020)
      addch('o');
   else addch('.');
   if (year & 0x0010)
      addch('o');
   else addch('.');
   if (year & 0x0008)
      addch('o');
   else addch('.');
   if (year & 0x0004)
      addch('o');
   else addch('.');
   if (year & 0x0002)
      addch('o');
   else addch('.');
   if (year & 0x0001)
      addch('o');
   else addch('.');
   if (iflpyr(year))
      mvaddch(0,25,'o');
   else mvaddch(0,25,'.');
   } /* putyear */

void putmon(int mon)
   {
   if (mon & 0x0008)
      mvaddch(1,18,'o');
   else mvaddch(1,18,'.');
   if (mon & 0x0004)
      addch('o');
   else addch('.');
   if (mon & 0x0002)
      addch('o');
   else addch('.');
   if (mon & 0x0001)
      addch('o');
   else addch('.');
   } /* putmon */

void putday(int day)
   {
   if (day & 0x0010)
      mvaddch(2,17,'o');
   else mvaddch(2,17,'.');
   if (day & 0x0008)
      addch('o');
   else addch('.');
   if (day & 0x0004)
      addch('o');
   else addch('.');
   if (day & 0x0002)
      addch('o');
   else addch('.');
   if (day & 0x0001)
      addch('o');
   else addch('.');
   } /* putday */

void putjul(int julday)
   {
   if (julday & 0x0100)
      mvaddch(3,13,'o');
   else mvaddch(3,13,'.');
   if (julday & 0x0080)
      addch('o');
   else addch('.');
   if (julday & 0x0040)
      addch('o');
   else addch('.');
   if (julday & 0x0020)
      addch('o');
   else addch('.');
   if (julday & 0x0010)
      addch('o');
   else addch('.');
   if (julday & 0x0008)
      addch('o');
   else addch('.');
   if (julday & 0x0004)
      addch('o');
   else addch('.');
   if (julday & 0x0002)
      addch('o');
   else addch('.');
   if (julday & 0x0001)
      addch('o');
   else addch('.');
   } /* putjul */

void putdow(int wday)
   {
   if (wday & 0x0004)
      mvaddch(3,25,'o');
   else mvaddch(3,25,'.');
   if (wday & 0x0002)
      addch('o');
   else addch('.');
   if (wday & 0x0001)
      addch('o');
   else addch('.');
   } /* putdow */

void puthour(int hour)
   {
   int i;
   i = hour % 12;
   if (i == 0) i = 12;
   if (i & 0x0008)
      mvaddch(5,18,'o');
   else mvaddch(5,18,'.');
   if (i & 0x0004)
      addch('o');
   else addch('.');
   if (i & 0x0002)
      addch('o');
   else addch('.');
   if (i & 0x0001)
      addch('o');
   else addch('.');
   if (hour > 11)
      mvaddch(5,25,'o');
   else mvaddch(5,25,'.');
   } /* puthour */

void putdst(int dst)
   {
   if (dst)
      mvaddch(5,28,'o');
   else mvaddch(5,28,'.');
   } /* putdst */

void putmin(int min)
   {
   if (min & 0x0020)
      mvaddch(6,16,'o');
   else mvaddch(6,16,'.');
   if (min & 0x0010)
      addch('o');
   else addch('.');
   if (min & 0x0008)
      addch('o');
   else addch('.');
   if (min & 0x0004)
      addch('o');
   else addch('.');
   if (min & 0x0002)
      addch('o');
   else addch('.');
   if (min & 0x0001)
      addch('o');
   else addch('.');
   } /* putmin */

void putsec(int sec)
   {
   if (sec & 0x0020)
      mvaddch(7,16,'o');
   else mvaddch(7,16,'.');
   if (sec & 0x0010)
      addch('o');
   else addch('.');
   if (sec & 0x0008)
      addch('o');
   else addch('.');
   if (sec & 0x0004)
      addch('o');
   else addch('.');
   if (sec & 0x0002)
      addch('o');
   else addch('.');
   if (sec & 0x0001)
      addch('o');
   else addch('.');
   } /* putsec */

void putascii(int year,int jul,int dst,char *str)
   {
   int iflpyr();
   char julian[128];
   /* ascii date */
   move(9,5);
   clrtoeol();
   addstr(str);
   if (jul != prevjul)
      {
      sprintf(julian,"Julian Day %d", jul);
      move(10,5);
      clrtoeol();
      addstr(julian);
      } /* if julian day changed */
   if (dst != prevdst)
      {
      if (dst)
         {
         move(11,5);
         clrtoeol();
         addstr("Daylight Saving");
         } /* if daylight saving */
      else
         {
         move(11,5);
         clrtoeol();
         addstr("Standard Time");
         } /* else standard time */
      } /* if daylight saving changed */
   if (year != prevyr)
      {
      if (iflpyr(year))
         {
         move(12,5);
         clrtoeol();
         addstr("Leap Year");
         } /* if leap year */
      else
         {
         move(12,5);
         clrtoeol();
         addstr("Non-leap Year");
         } /* else not leap year */
      } /* if daylight saving changed */
   } /* putascii */

void runclk()
   {
   int curryr;
   time_t now;
   struct tm *lcltm;
   char *str;
   while (runflg)
      {
      /* get date & time */
      time(&now);
      lcltm = (struct tm *) localtime(&now);
      curryr = lcltm->tm_year+1900;
      if (alfa)
         {
         str = (char *) asctime(lcltm);
         putascii(curryr,
            lcltm->tm_yday+1,
            lcltm->tm_isdst,
            str);
         } /* if alfa display on */
      if (curryr != prevyr)
         {
         putyear(curryr);
         prevyr = curryr;
         } /* if change */
      if (lcltm->tm_mon+1 != prevmon)
         {
         putmon(lcltm->tm_mon+1);
         prevmon = lcltm->tm_mon+1;
         } /* if change */
      if (lcltm->tm_mday != prevday)
         {
         putday(lcltm->tm_mday);
         prevday = lcltm->tm_mday;
         } /* if change */
      if (lcltm->tm_yday+1 != prevjul)
         {
         putjul(lcltm->tm_yday+1);
         prevjul = lcltm->tm_yday+1;
         } /* if change */
      if (lcltm->tm_wday+1 != prevdow)
         {
         putdow(lcltm->tm_wday+1);
         prevdow = lcltm->tm_wday+1;
         } /* if change */
      if (lcltm->tm_hour != prevhr)
         {
         puthour(lcltm->tm_hour);
         prevhr = lcltm->tm_hour;
         } /* if change */
      if (lcltm->tm_isdst != prevdst)
         {
         putdst(lcltm->tm_isdst);
         prevdst = lcltm->tm_isdst;
         } /* if change */
      if (lcltm->tm_min != prevmin)
         {
         putmin(lcltm->tm_min);
         prevmin = lcltm->tm_min;
         } /* if change */
      if (lcltm->tm_sec != prevsec)
         {
         putsec(lcltm->tm_sec);
         prevsec = lcltm->tm_sec;
         } /* if change */
      move(0,0);
      refresh();
      escape();
      if (runflg) sleep(1);
      } /* display iteration */
   } /* runclk */

int main()
   {
   prevyr = prevmon = prevday = prevjul = prevdow
      = prevhr = prevmin = prevsec = prevdst = -99999;
   initnc();
   alfa = 1;              /* default to alpha display */
   runflg = 1;
   runclk();
   endwin();
   return(0);
   } /* main */
